import { AbstractControl, FormArray, FormControl, FormGroup } from '@angular/forms';
import { ActivatedRouteSnapshot, Params } from '@angular/router';
import { DataMapper } from '@softeq/data-mappers';
import { Hash, Maybe } from '@softeq/types';
import { Observable } from 'rxjs';
/**
 * This interface defines tracker for subset of route query parameters. It is responsible for
 * <ul>
 * <li> tracking of route query parameters;
 * <li> tracking of filter changes;
 * <li> notifying about new query parameters
 * </ul>
 */
export interface RouteParamTracker<F> {
    /**
     * Stream of filter changes. As soon as filter has been changed this stream should emit new value.
     * If this tracker is registered as filter tracker, emitted value will be available
     * under {@link RouteTracker#filterChanges}
     */
    filterChanges: Observable<F>;
    /**
     * Stream of query parameters. When stream emits new query parameters, they are merged into URL of the current page.
     */
    paramChanges: Observable<Params>;
    /**
     * This method is called when query parameters of the current route has been modified.
     * It gets all current query parameters.
     * Typically implementation of this method should handle received parameters
     * and emit new filter (under {@link RouteParamTracker#filterChanges}).
     */
    mergeParams(params: Params): void;
    /**
     * This method allows to setup default filter. Main role of default filter is to reset query parameters.
     * @param defaultFilter
     */
    withDefault(defaultFilter: F): this;
}
/**
 * Manual tracker is used to immediately manage routable parameters
 */
export declare type ManualRouteParamTracker<F> = RouteParamTracker<F> & {
    setValue(filter: F): void;
};
/**
 * This interface defines typical pattern of criteria-based filtering.
 *
 * In typical scenario page having filter provides set of criterion (inputs, selects, check boxes)
 * and trigger button ('Search') to start searching.
 * This fits into this interface in the following way:
 * <ul>
 * <li> when user changes filter criteria => underlying filter is modified,
 *    but search is not triggered until filter is emitted by {@link Filterable#emitFilter};
 * <li> when user push trigger button ('Search'), page should call {@link Filterable#emitFilter} method
 *    to emit new value into {@link Filterable#filterChanges} stream;
 * <li> on the other hand, when query parameters has been modified {@link RouteTracker} finally
 *    calls {@link Filterable#setFilter} to notify implementation of {@link Filterable} interface about new filter.
 * </ul>
 */
export interface Filterable<F> {
    /**
     * Current filter
     */
    readonly currentFilter: F;
    /**
     * This stream emits new filter as soon as {@link Filterable#emitFilter} method is called.
     */
    filterChanges: Observable<F>;
    /**
     * Updates current filter
     */
    setFilter(filter: F): void;
    /**
     * Emits current filter.
     */
    emitFilter(): void;
}
/**
 * Interface to define {@link RouteParamTracker} based on {@link Observable}.
 */
export interface ObservableFilter<F> {
    filterChanges: Observable<F>;
    /** Name of query parameter this filter will be saved under */
    name?: string;
    dataMapper?: DataMapper<F, any>;
    setFilter?(filter: F): void;
}
/**
 * Interface to define plain {@link RouteParamTracker} based on {@link Observable}.
 * Plain tracker merges emitted filter into set of specified query parameters
 * (rather than single parameter as other trackers)
 */
export interface ObservablePlainFilter<F> {
    /** Set of filter fields to be merged as query parameters */
    names: string[];
    filterChanges: Observable<F>;
    /** Optional {@link DataMapper}s used to serialize/deserialize filter fields */
    dataMappers?: Hash<DataMapper<any, any>>;
    setFilter?(filter: F): void;
}
export declare function getRouteTrackerFilter<F>(snapshot: ActivatedRouteSnapshot, mapper?: DataMapper<F, any>): Maybe<F>;
export declare function getRouteTrackerFilter<F>(snapshot: ActivatedRouteSnapshot, paramName?: string, mapper?: DataMapper<F, any>): Maybe<F>;
/**
 * Defines {@link RouteParamTracker} for {@link Filterable} object. Filter coming from/to query parameters will be
 * deserialized/serialized using provided <code>dataMapper</code> correspondingly.
 * This method stores filter under the single query parameter (by default, 'filter')
 */
export declare function filterTracker<F>(filterable: Filterable<F>, dataMapper?: DataMapper<F, any>): RouteParamTracker<F>;
export declare function filterTracker<F>(queryParamName: string, filterable: Filterable<F>, dataMapper?: DataMapper<F, any>): RouteParamTracker<F>;
/**
 * Defines {@link RouteParamTracker} for {@link AbstractControl} object. Filter coming from/to query parameters will be
 * deserialized/serialized using provided <code>dataMapper</code> correspondingly.
 * This method stores filter under the single query parameter (by default, 'filter')
 */
export declare function controlTracker<F>(control: AbstractControl, dataMapper?: DataMapper<F, any>): RouteParamTracker<F>;
export declare function controlTracker<F>(queryParamName: string, control: AbstractControl, dataMapper?: DataMapper<F, any>): RouteParamTracker<F>;
/**
 * Merges {@link Filterable} behavior into provided ReactiveForms control
 */
export declare function filterableControl<F>(control: FormControl): FormControl & Filterable<F>;
export declare function filterableControl<F>(control: FormGroup): FormGroup & Filterable<F>;
export declare function filterableControl<F>(control: FormArray): FormArray & Filterable<F>;
/**
 * Creates manual tracker.
 * This method stores filter under the single query parameter (by default, 'filter')
 */
export declare function manualTracker<F>(dataMapper?: DataMapper<F, any>): ManualRouteParamTracker<F>;
export declare function manualTracker<F>(queryParamName: string, dataMapper?: DataMapper<F, any>): ManualRouteParamTracker<F>;
/**
 * Defines {@link RouteParamTracker} which updates query parameters from the given {@link Observable}.
 * If query parameters is changed tracker calls {@link ObservableFilter#setFilter} method
 * to allow developer to update a page stage.
 * Filter coming from/to query parameters will be deserialized/serialized using provided <code>dataMapper</code>.
 * This method stores filter under the single query parameter (by default, 'filter')
 */
export declare function observableFilterTracker<F>(oFilter: ObservableFilter<F>): RouteParamTracker<F>;
/**
 * Defines {@link RouteParamTracker} which updates query parameters from the given {@link Observable}.
 * If query parameters is changed tracker calls {@link ObservableFilter#setFilter} method
 * to allow developer to update a page stage.
 * Filter coming from/to query parameters will be deserialized/serialized using provided <code>dataMappers</code>.
 * This tracker tracks only parameters specified under <code>names</code> parameter.
 * Modified filter is merged into query parameters defined by <code>names</code> parameter.
 */
export declare function observablePlainFilterTracker<F>(oFilter: ObservablePlainFilter<F>): RouteParamTracker<F>;
