import { ActivatedRoute, Router } from '@angular/router';
import { Observable, Subscription } from 'rxjs';
import { RouteParamTracker } from './route-param-tracker';
export declare type RouteTracker1<F1> = RouteTracker<[F1]>;
export declare type RouteTracker2<F1, F2> = RouteTracker<[F1, F2]>;
export declare type RouteTracker3<F1, F2, F3> = RouteTracker<[F1, F2, F3]>;
export declare type RouteTracker4<F1, F2, F3, F4> = RouteTracker<[F1, F2, F3, F4]>;
/**
 * RouteTracker allows to
 * -- track changes of route query parameters;
 * -- update in-page data when route query parameters has changed;
 * -- update route query parameters when in-page data has been changed.
 *
 * <i>
 * Example:
 * Application has a page displaying list of entries filtered by parameters set on UI.
 * When one of filter parameters is changed => list of entries should be reloaded accordingly.
 * User can open one of entries in the separate page (leaving list of entries).
 * When he/she returns to the list (by pressing 'Back' browser's button)
 * it is convenient to have the latest parameters set on UI.
 *
 * Typical solution of this problem is to save all filters in query parameters of the page.
 * It allows to implement 'Back' behavior and gives 'bookmarkability' of such pages (at least).
 * </i>
 *
 * This class helps to solve similar problems.
 * In order to track and update query parameters RouteTracker registers {@link RouteParamTracker}s.
 * Each {@link RouteParamTracker} tracks and updates one or several query parameters.
 *
 * All registered {@link RouteParamTracker}s are divided into filter and effect parameter trackers.
 *
 * When query parameters tracked by filter parameter trackers are changed => {@link RouteTracker} emits the last filters
 * into {@link RouteTracker#filterChanges} stream. This allows to update page based on filter changes.
 *
 * Effect parameter trackers also tracks query parameters, but they do not notify {@link RouteTracker#filterChanges}
 * stream about the changes. This is the only difference.
 */
export declare class RouteTracker<F> {
    private router;
    private filterTrackers;
    private effectTrackers;
    private defaultFilter;
    private enableMergeParams;
    private subscription?;
    private filterChanges$$;
    constructor(router: Router, filterTrackers: RouteParamTracker<any>[], effectTrackers: RouteParamTracker<any>[], defaultFilter: F);
    /**
     * Returns true iff tracker listens for some ActivatedRoute
     */
    readonly listening: boolean;
    /**
     * Returns the latest filters tracked by filter {@link RouteParamTracker}s
     */
    readonly filterChanges: Observable<F>;
    /**
     * Constructs new {@link RouteTracker} with the given set of {@link RouteParamTracker}s as effect param trackers.
     * Set of filter {@link RouteParamTracker}s is not modified.
     */
    withEffects(...effectTrackers: RouteParamTracker<any>[]): RouteTracker<F>;
    /**
     * Starts listening for given {@link ActivatedRoute}. Although this method returns {@link Subscription}, generally
     * it is unnecessary to unsubscribe explicitly. When user leaves given route, subscription is destroyed automatically.
     */
    listen(route: ActivatedRoute): Subscription;
    /**
     * Dispose subscription for listened route.
     */
    unlisten(): void;
    /**
     * Merges given query parametes into current URL. If parameters do not change final URL this method returns undefined.
     */
    private mergeQueryParams;
}
